// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
import React from 'react';
import { VDomRenderer, VDomModel } from '@jupyterlab/apputils';
import { Text } from '@jupyterlab/coreutils';
import { interactiveItem, TextItem } from '..';
import { JSONExt } from '@lumino/coreutils';
/**
 * A pure functional component for rendering kernel status.
 */
function KernelStatusComponent(props) {
    var _a;
    return (React.createElement(TextItem, { onClick: props.handleClick, source: `${props.kernelName} | ${Text.titleCase((_a = props.status, (_a !== null && _a !== void 0 ? _a : 'undefined')))}`, title: `Change kernel for ${props.activityName}` }));
}
/**
 * A VDomRenderer widget for displaying the status of a kernel.
 */
export class KernelStatus extends VDomRenderer {
    /**
     * Construct the kernel status widget.
     */
    constructor(opts) {
        super(new KernelStatus.Model());
        this._handleClick = opts.onClick;
        this.addClass(interactiveItem);
    }
    /**
     * Render the kernel status item.
     */
    render() {
        if (this.model === null) {
            return null;
        }
        else {
            return (React.createElement(KernelStatusComponent, { status: this.model.status, kernelName: this.model.kernelName, activityName: this.model.activityName, handleClick: this._handleClick }));
        }
    }
}
/**
 * A namespace for KernelStatus statics.
 */
(function (KernelStatus) {
    /**
     * A VDomModel for the kernel status indicator.
     */
    class Model extends VDomModel {
        constructor() {
            super(...arguments);
            /**
             * React to changes to the kernel status.
             */
            this._onKernelStatusChanged = () => {
                var _a;
                this._kernelStatus = (_a = this._sessionContext) === null || _a === void 0 ? void 0 : _a.kernelDisplayStatus;
                this.stateChanged.emit(void 0);
            };
            /**
             * React to changes in the kernel.
             */
            this._onKernelChanged = (_sessionContext, change) => {
                var _a;
                const oldState = this._getAllState();
                const { newValue } = change;
                if (newValue !== null) {
                    // sync setting of status and display name
                    this._kernelStatus = (_a = this._sessionContext) === null || _a === void 0 ? void 0 : _a.kernelDisplayStatus;
                    this._kernelName = _sessionContext.kernelDisplayName;
                    this._triggerChange(oldState, this._getAllState());
                }
                else {
                    this._kernelStatus = '';
                    this._kernelName = 'No Kernel!';
                    this._triggerChange(oldState, this._getAllState());
                }
            };
            this._activityName = 'activity';
            this._kernelName = 'No Kernel!';
            this._kernelStatus = '';
            this._sessionContext = null;
        }
        /**
         * The name of the kernel.
         */
        get kernelName() {
            return this._kernelName;
        }
        /**
         * The current status of the kernel.
         */
        get status() {
            return this._kernelStatus;
        }
        /**
         * A display name for the activity.
         */
        get activityName() {
            return this._activityName;
        }
        set activityName(val) {
            const oldVal = this._activityName;
            if (oldVal === val) {
                return;
            }
            this._activityName = val;
            this.stateChanged.emit(void 0);
        }
        /**
         * The current client session associated with the kernel status indicator.
         */
        get sessionContext() {
            return this._sessionContext;
        }
        set sessionContext(sessionContext) {
            var _a, _b, _c, _d, _e, _f, _g, _h;
            (_a = this._sessionContext) === null || _a === void 0 ? void 0 : _a.statusChanged.disconnect(this._onKernelStatusChanged);
            (_b = this._sessionContext) === null || _b === void 0 ? void 0 : _b.kernelChanged.disconnect(this._onKernelChanged);
            const oldState = this._getAllState();
            this._sessionContext = sessionContext;
            this._kernelStatus = (_c = sessionContext) === null || _c === void 0 ? void 0 : _c.kernelDisplayStatus;
            this._kernelName = (_e = (_d = sessionContext) === null || _d === void 0 ? void 0 : _d.kernelDisplayName, (_e !== null && _e !== void 0 ? _e : 'No Kernel!'));
            (_f = sessionContext) === null || _f === void 0 ? void 0 : _f.statusChanged.connect(this._onKernelStatusChanged, this);
            (_g = sessionContext) === null || _g === void 0 ? void 0 : _g.connectionStatusChanged.connect(this._onKernelStatusChanged, this);
            (_h = sessionContext) === null || _h === void 0 ? void 0 : _h.kernelChanged.connect(this._onKernelChanged, this);
            this._triggerChange(oldState, this._getAllState());
        }
        _getAllState() {
            return [this._kernelName, this._kernelStatus, this._activityName];
        }
        _triggerChange(oldState, newState) {
            if (JSONExt.deepEqual(oldState, newState)) {
                this.stateChanged.emit(void 0);
            }
        }
    }
    KernelStatus.Model = Model;
})(KernelStatus || (KernelStatus = {}));
//# sourceMappingURL=kernelStatus.js.map